/*
 * Created on 2003-8-23
 */
package net.zhoufeng;

import java.io.File;
import java.util.Iterator;
import java.util.Vector;

import de.vdheide.mp3.ID3;
import de.vdheide.mp3.ID3v2;
import de.vdheide.mp3.ID3v2Frame;

/**
 * @author zf
 *
 */
public class ID3iconv {

	public static void main(String[] args) {
		if (args.length == 0) {
			usage();
			return;			
		}
		ID3iconv encoder = new ID3iconv();
		for (int i = 0; i < args.length; i++) try {
			info ("Converting "+args[i]);
			encoder.convert (new File(args[i]), "GBK");
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	/**
	 * @param string
	 */
	private static void info(String string) {
		System.out.println(string);
	}

	ID3 id3;
	ID3v2 id3v2;

	
	private void addFrame(String id, String content) throws Exception {
		ID3v2Frame frame;
		debug (id+": "+content);
		byte[] newbuf = content.getBytes("UnicodeLittle");
		if (newbuf.length == 0)
			return;
		byte[] newbuf2 = new byte[newbuf.length+3];
		System.arraycopy(newbuf, 0, newbuf2, 1, newbuf.length);
		newbuf2[newbuf2.length-2]=newbuf2[newbuf2.length-1]=0;
		newbuf2[0] = 1;		// encoding: utf-16

		frame = new ID3v2Frame (id, newbuf2,
			false, false, false, ID3v2Frame.NO_COMPRESSION, (byte)0, (byte)0);
		id3v2.addFrame(frame);
	}

	/**
	 * Convert the ID3 tag from any encoding to UTF-8.  If the original file
	 * contains ID3 v1 tags, it is converted to ID3v2 in order to support Unicode.
	 * @param file
	 */
	private void convert(File file, String encoding) throws Exception{
		id3 = new ID3(file);
		id3.encoding = encoding;
		id3v2 = new ID3v2(file);

		boolean hasv1 = id3.checkForTag();
		boolean hasv2 = id3v2.hasTag();
		
		if (hasv2) try {
			id3v2.getFrames();
		} catch (Exception e) {
			debug ("Cannot get v2 frames, assuming no v2 tag.");
			hasv2 = false;
		}
		
		if (hasv1 && !hasv2) {
			// convert ID3v1 to ID3v2
			info ("Converting id3v1 tag to id3v2 format.");
			addFrame("TALB", id3.getAlbum());
			addFrame("TOPE", id3.getArtist());
			addFrame("TPE1", id3.getArtist());
			addFrame("COMM", id3.getComment());
			addFrame("TIT2", id3.getTitle());
			addFrame("TORY", id3.getYear());
			int i = id3.getGenre();
			if (i >= 0 && i < genreString.length)
				addFrame("TCON", genreString[i]);
			else
				addFrame("TCON", "unknown");
			addFrame("TRCK", id3.getTrack()+"");
			id3.removeTag();
			id3v2.update();
		} else if (hasv1) {
			// convert all text frames
			info ("Reencoding id3v2 tag");
			boolean updated = false;
			Vector frames = id3v2.getFrames();
			if (frames != null && frames.size() > 0) {
				for (Iterator iter = frames.iterator(); iter.hasNext();) {
					ID3v2Frame frame = (ID3v2Frame) iter.next();
					if (frame.getID().startsWith("T")) {
						byte[] buf = frame.getContent();
						if (buf.length > 1 && buf[0] == 0) {
							String s = new String(buf, 1, buf.length-1, encoding);
							debug(frame.getID() + ": "+s);
							byte[] newbuf = s.getBytes("UTF-16");
							byte[] newbuf2 = new byte[newbuf.length+3];
							System.arraycopy(newbuf, 0, newbuf2, 1, newbuf.length);
							newbuf2[newbuf2.length-2]=newbuf2[newbuf2.length-1]=0;
							newbuf2[0] = 1;		// encoding: utf-16
							frame.setContent(newbuf2);
							updated = true;
						}
					} else {
						debug ("No action for frame: "+frame.getID());
					}
				}
				if (updated) {
					id3v2.touch();
					id3v2.update();
				}
			}
			
		} else {
			info ("File has no id3 tag, skipping!");
		}
	}

	/**
	 * @param string
	 */
	private void debug(String string) {
		System.err.println(string);
	}

	/**
	 * 
	 */
	private static void usage() {
		System.out.println("ID3iconv - convert ID3 tags from machine default encoding "
					+ "to unicode and store them using ID3v2.3 format.\n"
					+ "Pass mp3 file names as arguments.");		
	}
	
	String genreString[] = {"Blues"
		,  "Classic Rock"
		,  "Country"
		,  "Dance"
		,  "Disco"
		,  "Funk"
		,  "Grunge"
		,  "Hip-Hop"
		,  "Jazz"
		,  "Metal"
		,  "New Age"
		,  "Oldies"
		,  "Other"
		,  "Pop"
		,  "R&B"
		,  "Rap"
		,  "Reggae"
		,  "Rock"
		,  "Techno"
		,  "Industrial"
		,  "Alternative"
		,  "Ska"
		,  "Death Metal"
		,  "Pranks"
		,  "Soundtrack"
		,  "Euro-Techno"
		,  "Ambient"
		,  "Trip-Hop"
		,  "Vocal"
		,  "Jazz Funk"
		,  "Fusion"
		,  "Trance"
		,  "Classical"
		,  "Instrumental"
		,  "Acid"
		,  "House"
		,  "Game"
		,  "Sound Clip"
		,  "Gospel"
		,  "Noise"
		,  "Alternative Rock"
		,  "Bass"
		,  "Soul"
		,  "Punk"
		,  "Space"
		,  "Meditative"
		,  "Instrumental Pop"
		,  "Instrumental Rock"
		,  "Ethnic"
		,  "Gothic"
		,  "Darkwave"
		,  "Techno-Industrial"
		,  "Electronic"
		,  "Pop-Folk"
		,  "Eurodance"
		,  "Dream"
		,  "Southern Rock"
		,  "Comedy"
		,  "Cult"
		,  "Gangsta"
		,  "Top 40"
		,  "Christian Rap"
		,  "Pop/Funk"
		,  "Jungle"
		,  "Native American"
		,  "Cabaret"
		,  "New Wave"
		,  "Psychadelic"
		,  "Rave"
		,  "Showtunes"
		,  "Trailer"
		,  "Lo-Fi"
		,  "Tribal"
		,  "Acid Punk"
		,  "Acid Jazz"
		,  "Polka"
		,  "Retro"
		,  "Musical"
		,  "Rock & Roll"
		,  "Hard Rock"
		,  "Folk"
		,  "Folk/Rock"
		,  "National Folk"
		,  "Swing"
		,  "Fast Fusion"
		,  "Bebob"
		,  "Latin"
		,  "Revival"
		,  "Celtic"
		,  "Bluegrass"
		,  "Avantgarde"
		,  "Gothic Rock"
		,  "Progressive Rock"
		,  "Psychedelic Rock"
		,  "Symphonic Rock"
		,  "Slow Rock"
		,  "Big Band"
		,  "Chorus"
		,  "Easy Listening"
		,  "Acoustic"
		,  "Humour"
		,  "Speech"
		,  "Chanson"
		,  "Opera"
		,  "Chamber Music"
		,  "Sonata"
		,  "Symphony"
		,  "Booty Bass"
		,  "Primus"
		,  "Porn Groove"
		,  "Satire"
		,  "Slow Jam"
		,  "Club"
		,  "Tango"
		,  "Samba"
		,  "Folklore"
		,  "Ballad"
		,  "Power Ballad"
		,  "Rhythmic Soul"
		,  "Freestyle"
		,  "Duet"
		,  "Punk Rock"
		,  "Drum Solo"
		,  "A Capella"
		,  "Euro-House"
		,  "Dance Hall"
		,  "Goa"
		,  "Drum & Bass"
		,  "Club-House"
		,  "Hardcore"
		,  "Terror"
		,  "Indie"
		,  "BritPop"
		,  "Negerpunk"
		,  "Polsk Punk"
		,  "Beat"
		,  "Christian Gangsta Rap"
		,  "Heavy Metal"
		,  "Black Metal"
		,  "Crossover"
		,  "Contemporary Christian"
		,  "Christian Rock"
		,  "Merengue"
		,  "Salsa"
		,  "Thrash Metal"
		,  "Anime"
		,  "JPop"
		,  "Synthpop"
	};
	
}
